/* Project:     OSLib
 * Description: The OS Construction Kit
 * Date:                1.6.2000
 * Idea by:             Luca Abeni & Gerardo Lamastra
 *
 * OSLin is an SO project aimed at developing a common, easy-to-use
 * low-level infrastructure for developing OS kernels and Embedded
 * Applications; it partially derives from the HARTIK project but it
 * currently is independently developed.
 *
 * OSLib is distributed under GPL License, and some of its code has
 * been derived from the Linux kernel source; also some important
 * ideas come from studying the DJGPP go32 extender.
 *
 * We acknowledge the Linux Community, Free Software Foundation,
 * D.J. Delorie and all the other developers who believe in the
 * freedom of software and ideas.
 *
 * For legalese, check out the included GPL license.
 */

/*	CPU detection code	*/

.title	"CPU2.S"

#include <ll/i386/linkage.h>
#include <ll/i386/defs.h>

/*
 * The following code has been extracted by Intel AP-485
 * Application Note: Intel Processor Identification with CPUID instruction
 */

.data

ASMFILE(CPU)

.globl	SYMBOL_NAME(X86_cpu)
.globl	SYMBOL_NAME(X86_fpu)
.globl	SYMBOL_NAME(X86_v86Flag)
.globl	SYMBOL_NAME(X86_cpuidFlag)
.globl	SYMBOL_NAME(X86_vendor)
.globl	SYMBOL_NAME(X86_signature)
.globl	SYMBOL_NAME(X86_features1)
.globl	SYMBOL_NAME(X86_features2)
.globl	SYMBOL_NAME(X86_features3)

SYMBOL_NAME_LABEL(X86_cpu)		.byte	0
SYMBOL_NAME_LABEL(X86_fpu)		.byte	0
SYMBOL_NAME_LABEL(X86_v86Flag)		.byte	0
SYMBOL_NAME_LABEL(X86_cpuidFlag) 	.byte	0
SYMBOL_NAME_LABEL(X86_vendor)		.space	13,0
SYMBOL_NAME_LABEL(X86_signature)	.int	0
SYMBOL_NAME_LABEL(X86_features1)	.int	0
SYMBOL_NAME_LABEL(X86_features2)	.int	0
SYMBOL_NAME_LABEL(X86_features3)	.int	0
fpu_status:	.word	0

.text

.globl SYMBOL_NAME(X86_get_CPU)
.globl SYMBOL_NAME(X86_get_FPU)

SYMBOL_NAME_LABEL(X86_get_CPU)	
		pushfl
		popl	%eax
		movl	%eax,%ecx
		xorl	$0x40000,%eax
		pushl	%eax
		popfl
		pushfl
		popl	%eax
		xorl	%ecx,%eax
		movb	$3,SYMBOL_NAME(X86_cpu)
		jz	endProc
		
		pushl	%ecx
		popfl	

chk486:		movl	$4,SYMBOL_NAME(X86_cpu)
		movl	%ecx,%eax
		xorl	$0x200000,%eax
		pushl	%eax
		popfl
		pushfl
		popl	%eax
		xorl	%ecx,%eax
		je	endProc
		
		movl	$1,SYMBOL_NAME(X86_cpuidFlag)
		pushl	%ebx
		pushl	%esi
		pushl	%edi
		movl	$0,%eax
		.byte 0x0F,0xA2
/*		cpuid */
		movl	%ebx,SYMBOL_NAME(X86_vendor)
		movl	%edx,SYMBOL_NAME(X86_vendor)+4
		movl	%ecx,SYMBOL_NAME(X86_vendor)+8
		
		cmpl	$1,%eax
		jl	endProc1
		movl	$1,%eax
		.byte 0x0F,0xA2
/*		cpuid */
		movl	%eax,SYMBOL_NAME(X86_signature)
		movl	%ebx,SYMBOL_NAME(X86_features1)
		movl	%ecx,SYMBOL_NAME(X86_features2)
		movl	%edx,SYMBOL_NAME(X86_features3)
		shrl	$8,%eax
		andl	$0x0F,%eax
		movb	%al,SYMBOL_NAME(X86_cpu)

endProc1:	popl	%edi
		popl	%esi
		popl	%ebx

endProc:	ret

SYMBOL_NAME_LABEL(X86_get_FPU)	
		fninit
		movw	$0x5a5a,fpu_status
		fnstsw	fpu_status
		movw	fpu_status,%ax
		cmpb	$0,%al
		movb	$0,SYMBOL_NAME(X86_fpu)
		jne	endFPUProc

chkCW:		fnstcw 	fpu_status
		movw	fpu_status,%ax
		andw	$0x103f,%ax
		cmpw	$0x03F,%ax
		movb	$0,SYMBOL_NAME(X86_fpu)
		jne	endFPUProc
		movb	$1,SYMBOL_NAME(X86_fpu)
		
chkInf:		cmpb	$3,SYMBOL_NAME(X86_cpu)
		jle	endFPUProc
		fld1
		fldz
		fdiv
		fld	%st
		fchs
		fcompp
		fstsw	fpu_status
		movb	$2,SYMBOL_NAME(X86_fpu)
		sahf
		jz	endFPUProc
		movb	$3,SYMBOL_NAME(X86_fpu)
endFPUProc:	ret
